/*  <title>Geometry</title>

	ETGeometry.m
	
	<abstract>Geometry utility functions and constants.</abstract>
 
	Copyright (C) 2008 Quentin Mathe
 
	Author:  Quentin Mathe <qmathe@club-internet.fr>
	Date:  December 2008
 
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	* Redistributions of source code must retain the above copyright notice,
	  this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright notice,
	  this list of conditions and the following disclaimer in the documentation
	  and/or other materials provided with the distribution.
	* Neither the name of the Etoile project nor the names of its contributors
	  may be used to endorse or promote products derived from this software
	  without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
	THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "ETGeometry.h"
#import <EtoileFoundation/Macros.h>
#include <float.h>

const NSPoint ETNullPoint = {FLT_MIN, FLT_MIN};
const NSSize ETNullSize = {FLT_MIN, FLT_MIN};
const NSRect ETNullRect = {{FLT_MIN, FLT_MIN}, {FLT_MIN, FLT_MIN}};

typedef NSRect (*RectIMP)(id, SEL);

/** Returns an union rect computed by iterating over itemArray and unionning the 
rects returned by the objects which must all respond to rectSelector.

itemArray must only contain objects of the same type.

A zero rect is returned when itemArray is empty. */
NSRect ETUnionRectWithObjectsAndSelector(NSArray *itemArray, SEL rectSelector)
{
	if ([itemArray count] == 0)
		return NSZeroRect;

	NSRect rect = NSZeroRect;
	RectIMP rectFunction = (RectIMP)[[itemArray objectAtIndex: 0] methodForSelector: rectSelector];

	FOREACHI(itemArray, item)
	{
		rect = NSUnionRect(rect, rectFunction(item, rectSelector));
	}

	return rect;
}
